@@ -6,6 +6,10 @@ type DescribeAclsRequest struct {
 	AclFilter
 }
 
+func (d *DescribeAclsRequest) setVersion(v int16) {
+	d.Version = int(v)
+}
+
 func (d *DescribeAclsRequest) encode(pe packetEncoder) error {
 	d.AclFilter.Version = d.Version
 	return d.AclFilter.encode(pe)
@@ -18,7 +22,7 @@ func (d *DescribeAclsRequest) decode(pd packetDecoder, version int16) (err error
 }
 
 func (d *DescribeAclsRequest) key() int16 {
-	return 29
+	return apiKeyDescribeAcls
 }
 
 func (d *DescribeAclsRequest) version() int16 {
