@@ -7,12 +7,21 @@ import (
 	"time"
 )
 
-var deleteTopicsRequest = []byte{
-	0, 0, 0, 2,
-	0, 5, 't', 'o', 'p', 'i', 'c',
-	0, 5, 'o', 't', 'h', 'e', 'r',
-	0, 0, 0, 100,
-}
+var (
+	deleteTopicsRequest = []byte{
+		0, 0, 0, 2,
+		0, 5, 't', 'o', 'p', 'i', 'c',
+		0, 5, 'o', 't', 'h', 'e', 'r',
+		0, 0, 0, 100,
+	}
+	deleteTopicsRequestV4 = []byte{
+		3,
+		6, 't', 'o', 'p', 'i', 'c',
+		6, 'o', 't', 'h', 'e', 'r',
+		0, 0, 0, 100,
+		0, // empty tagged fields
+	}
+)
 
 func TestDeleteTopicsRequestV0(t *testing.T) {
 	req := &DeleteTopicsRequest{
@@ -33,3 +42,13 @@ func TestDeleteTopicsRequestV1(t *testing.T) {
 
 	testRequest(t, "", req, deleteTopicsRequest)
 }
+
+func TestDeleteTopicsRequestV4(t *testing.T) {
+	req := &DeleteTopicsRequest{
+		Version: 4,
+		Topics:  []string{"topic", "other"},
+		Timeout: 100 * time.Millisecond,
+	}
+
+	testRequest(t, "", req, deleteTopicsRequestV4)
+}
