@@ -8,6 +8,10 @@ type EndTxnRequest struct {
 	TransactionResult bool
 }
 
+func (a *EndTxnRequest) setVersion(v int16) {
+	a.Version = v
+}
+
 func (a *EndTxnRequest) encode(pe packetEncoder) error {
 	if err := pe.putString(a.TransactionalID); err != nil {
 		return err
@@ -39,7 +43,7 @@ func (a *EndTxnRequest) decode(pd packetDecoder, version int16) (err error) {
 }
 
 func (a *EndTxnRequest) key() int16 {
-	return 26
+	return apiKeyEndTxn
 }
 
 func (a *EndTxnRequest) version() int16 {
