@@ -8,11 +8,16 @@ type HeartbeatResponse struct {
 	Err          KError
 }
 
+func (r *HeartbeatResponse) setVersion(v int16) {
+	r.Version = v
+}
+
 func (r *HeartbeatResponse) encode(pe packetEncoder) error {
 	if r.Version >= 1 {
 		pe.putInt32(r.ThrottleTime)
 	}
-	pe.putInt16(int16(r.Err))
+	pe.putKError(r.Err)
+	pe.putEmptyTaggedFieldArray()
 	return nil
 }
 
@@ -24,17 +29,17 @@ func (r *HeartbeatResponse) decode(pd packetDecoder, version int16) error {
 			return err
 		}
 	}
-	kerr, err := pd.getInt16()
+	r.Err, err = pd.getKError()
 	if err != nil {
 		return err
 	}
-	r.Err = KError(kerr)
 
-	return nil
+	_, err = pd.getEmptyTaggedFieldArray()
+	return err
 }
 
 func (r *HeartbeatResponse) key() int16 {
-	return 12
+	return apiKeyHeartbeat
 }
 
 func (r *HeartbeatResponse) version() int16 {
@@ -42,15 +47,28 @@ func (r *HeartbeatResponse) version() int16 {
 }
 
 func (r *HeartbeatResponse) headerVersion() int16 {
+	if r.Version >= 4 {
+		return 1
+	}
 	return 0
 }
 
 func (r *HeartbeatResponse) isValidVersion() bool {
-	return r.Version >= 0 && r.Version <= 3
+	return r.Version >= 0 && r.Version <= 4
+}
+
+func (r *HeartbeatResponse) isFlexible() bool {
+	return r.isFlexibleVersion(r.Version)
+}
+
+func (r *HeartbeatResponse) isFlexibleVersion(version int16) bool {
+	return version >= 4
 }
 
 func (r *HeartbeatResponse) requiredVersion() KafkaVersion {
 	switch r.Version {
+	case 4:
+		return V2_4_0_0
 	case 3:
 		return V2_3_0_0
 	case 2:
