@@ -19,6 +19,16 @@ var (
 		0, 0, 0, 100,
 		0, byte(ErrFencedInstancedId),
 	}
+	heartbeatResponseNoError_V4 = []byte{
+		0, 0, 0, 100,
+		0, 0,
+		0, // empty tagged fields
+	}
+	heartbeatResponseError_V4 = []byte{
+		0, 0, 0, 100,
+		0, byte(ErrFencedInstancedId),
+		0, // empty tagged fields
+	}
 )
 
 func TestHeartbeatResponse(t *testing.T) {
@@ -57,6 +67,26 @@ func TestHeartbeatResponse(t *testing.T) {
 				ThrottleTime: 100,
 			},
 		},
+		{
+			"v4_noErr",
+			4,
+			heartbeatResponseNoError_V4,
+			&HeartbeatResponse{
+				Version:      4,
+				Err:          ErrNoError,
+				ThrottleTime: 100,
+			},
+		},
+		{
+			"v4_Err",
+			4,
+			heartbeatResponseError_V4,
+			&HeartbeatResponse{
+				Version:      4,
+				Err:          ErrFencedInstancedId,
+				ThrottleTime: 100,
+			},
+		},
 	}
 	for _, c := range tests {
 		testEncodable(t, c.CaseName, c.Message, c.MessageBytes)
