@@ -20,6 +20,17 @@ var (
 		0, 4, 'm', 'i', 'd', '2', // MemberId
 		0, 3, 'g', 'i', 'd', // GroupInstanceId
 	}
+	basicLeaveGroupRequestV4 = []byte{
+		4, 'f', 'o', 'o',
+		3,                     // Two Member
+		5, 'm', 'i', 'd', '1', // MemberId
+		0,                     // GroupInstanceId  nil
+		0,                     // empty tagged fields
+		5, 'm', 'i', 'd', '2', // MemberId
+		4, 'g', 'i', 'd', // GroupInstanceId
+		0, // empty tagged fields
+		0, // empty tagged fields
+	}
 )
 
 func TestLeaveGroupRequest(t *testing.T) {
@@ -53,6 +64,19 @@ func TestLeaveGroupRequest(t *testing.T) {
 				},
 			},
 		},
+		{
+			"v4",
+			4,
+			basicLeaveGroupRequestV4,
+			&LeaveGroupRequest{
+				Version: 4,
+				GroupId: "foo",
+				Members: []MemberIdentity{
+					{"mid1", nil},
+					{"mid2", &groupInstanceId},
+				},
+			},
+		},
 	}
 	for _, c := range tests {
 		request := new(LeaveGroupRequest)
